// This file is generated by kconfig_compiler_kf6 from kwintouchscreensettings.kcfg.
// All changes you do to this file will be lost.
#ifndef KWIN_KWINTOUCHSCREENSETTINGS_H
#define KWIN_KWINTOUCHSCREENSETTINGS_H

#include "effect/globals.h"

#include <qglobal.h>
#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

namespace KWin {

class KWinTouchScreenSettings : public KConfigSkeleton
{
  public:

    KWinTouchScreenSettings( QObject *parent = nullptr );
    ~KWinTouchScreenSettings() override;

    /**
      Set Top
    */
    void setTop( const QString & v )
    {
      if (!isTopImmutable())
        mTop = v;
    }

    /**
      Get Top
    */
    QString top() const
    {
      return mTop;
    }

    /**
      Is Top Immutable
    */
    bool isTopImmutable() const
    {
      return isImmutable( QStringLiteral( "Top" ) );
    }

    /**
      Get Top default value
    */
    QString defaultTopValue() const
    {
        return defaultTopValue_helper();
    }

    /**
      Set Right
    */
    void setRight( const QString & v )
    {
      if (!isRightImmutable())
        mRight = v;
    }

    /**
      Get Right
    */
    QString right() const
    {
      return mRight;
    }

    /**
      Is Right Immutable
    */
    bool isRightImmutable() const
    {
      return isImmutable( QStringLiteral( "Right" ) );
    }

    /**
      Get Right default value
    */
    QString defaultRightValue() const
    {
        return defaultRightValue_helper();
    }

    /**
      Set Bottom
    */
    void setBottom( const QString & v )
    {
      if (!isBottomImmutable())
        mBottom = v;
    }

    /**
      Get Bottom
    */
    QString bottom() const
    {
      return mBottom;
    }

    /**
      Is Bottom Immutable
    */
    bool isBottomImmutable() const
    {
      return isImmutable( QStringLiteral( "Bottom" ) );
    }

    /**
      Get Bottom default value
    */
    QString defaultBottomValue() const
    {
        return defaultBottomValue_helper();
    }

    /**
      Set Left
    */
    void setLeft( const QString & v )
    {
      if (!isLeftImmutable())
        mLeft = v;
    }

    /**
      Get Left
    */
    QString left() const
    {
      return mLeft;
    }

    /**
      Is Left Immutable
    */
    bool isLeftImmutable() const
    {
      return isImmutable( QStringLiteral( "Left" ) );
    }

    /**
      Get Left default value
    */
    QString defaultLeftValue() const
    {
        return defaultLeftValue_helper();
    }

    /**
      Set TouchBorderActivateAll
    */
    void setTouchBorderActivateAll( const QList<int> & v )
    {
      if (!isTouchBorderActivateAllImmutable())
        mTouchBorderActivateAll = v;
    }

    /**
      Get TouchBorderActivateAll
    */
    QList<int> touchBorderActivateAll() const
    {
      return mTouchBorderActivateAll;
    }

    /**
      Is TouchBorderActivateAll Immutable
    */
    bool isTouchBorderActivateAllImmutable() const
    {
      return isImmutable( QStringLiteral( "TouchBorderActivateAll" ) );
    }

    /**
      Get TouchBorderActivateAll default value
    */
    QList<int> defaultTouchBorderActivateAllValue() const
    {
        return defaultTouchBorderActivateAllValue_helper();
    }

    /**
      Set TouchBorderActivate
    */
    void setTouchBorderActivatePresentWindows( const QList<int> & v )
    {
      if (!isTouchBorderActivatePresentWindowsImmutable())
        mTouchBorderActivatePresentWindows = v;
    }

    /**
      Get TouchBorderActivate
    */
    QList<int> touchBorderActivatePresentWindows() const
    {
      return mTouchBorderActivatePresentWindows;
    }

    /**
      Is TouchBorderActivate Immutable
    */
    bool isTouchBorderActivatePresentWindowsImmutable() const
    {
      return isImmutable( QStringLiteral( "TouchBorderActivatePresentWindows" ) );
    }

    /**
      Get TouchBorderActivate default value
    */
    QList<int> defaultTouchBorderActivatePresentWindowsValue() const
    {
        return defaultTouchBorderActivatePresentWindowsValue_helper();
    }

    /**
      Set TouchBorderActivateClass
    */
    void setTouchBorderActivateClass( const QList<int> & v )
    {
      if (!isTouchBorderActivateClassImmutable())
        mTouchBorderActivateClass = v;
    }

    /**
      Get TouchBorderActivateClass
    */
    QList<int> touchBorderActivateClass() const
    {
      return mTouchBorderActivateClass;
    }

    /**
      Is TouchBorderActivateClass Immutable
    */
    bool isTouchBorderActivateClassImmutable() const
    {
      return isImmutable( QStringLiteral( "TouchBorderActivateClass" ) );
    }

    /**
      Get TouchBorderActivateClass default value
    */
    QList<int> defaultTouchBorderActivateClassValue() const
    {
        return defaultTouchBorderActivateClassValue_helper();
    }

    /**
      Set TouchBorderActivate
    */
    void setTouchBorderActivateOverview( const QList<int> & v )
    {
      if (!isTouchBorderActivateOverviewImmutable())
        mTouchBorderActivateOverview = v;
    }

    /**
      Get TouchBorderActivate
    */
    QList<int> touchBorderActivateOverview() const
    {
      return mTouchBorderActivateOverview;
    }

    /**
      Is TouchBorderActivate Immutable
    */
    bool isTouchBorderActivateOverviewImmutable() const
    {
      return isImmutable( QStringLiteral( "TouchBorderActivateOverview" ) );
    }

    /**
      Get TouchBorderActivate default value
    */
    QList<int> defaultTouchBorderActivateOverviewValue() const
    {
        return defaultTouchBorderActivateOverviewValue_helper();
    }

    /**
      Set GridTouchBorderActivate
    */
    void setTouchBorderActivateGrid( const QList<int> & v )
    {
      if (!isTouchBorderActivateGridImmutable())
        mTouchBorderActivateGrid = v;
    }

    /**
      Get GridTouchBorderActivate
    */
    QList<int> touchBorderActivateGrid() const
    {
      return mTouchBorderActivateGrid;
    }

    /**
      Is GridTouchBorderActivate Immutable
    */
    bool isTouchBorderActivateGridImmutable() const
    {
      return isImmutable( QStringLiteral( "TouchBorderActivateGrid" ) );
    }

    /**
      Get GridTouchBorderActivate default value
    */
    QList<int> defaultTouchBorderActivateGridValue() const
    {
        return defaultTouchBorderActivateGridValue_helper();
    }

    /**
      Set TouchBorderActivate
    */
    void setTouchBorderActivateTabBox( const QList<int> & v )
    {
      if (!isTouchBorderActivateTabBoxImmutable())
        mTouchBorderActivateTabBox = v;
    }

    /**
      Get TouchBorderActivate
    */
    QList<int> touchBorderActivateTabBox() const
    {
      return mTouchBorderActivateTabBox;
    }

    /**
      Is TouchBorderActivate Immutable
    */
    bool isTouchBorderActivateTabBoxImmutable() const
    {
      return isImmutable( QStringLiteral( "TouchBorderActivateTabBox" ) );
    }

    /**
      Get TouchBorderActivate default value
    */
    QList<int> defaultTouchBorderActivateTabBoxValue() const
    {
        return defaultTouchBorderActivateTabBoxValue_helper();
    }

    /**
      Set TouchBorderAlternativeActivate
    */
    void setTouchBorderAlternativeActivate( const QList<int> & v )
    {
      if (!isTouchBorderAlternativeActivateImmutable())
        mTouchBorderAlternativeActivate = v;
    }

    /**
      Get TouchBorderAlternativeActivate
    */
    QList<int> touchBorderAlternativeActivate() const
    {
      return mTouchBorderAlternativeActivate;
    }

    /**
      Is TouchBorderAlternativeActivate Immutable
    */
    bool isTouchBorderAlternativeActivateImmutable() const
    {
      return isImmutable( QStringLiteral( "TouchBorderAlternativeActivate" ) );
    }

    /**
      Get TouchBorderAlternativeActivate default value
    */
    QList<int> defaultTouchBorderAlternativeActivateValue() const
    {
        return defaultTouchBorderAlternativeActivateValue_helper();
    }

  protected:

    // TouchEdges
    QString mTop;
    QString defaultTopValue_helper() const;
    QString mRight;
    QString defaultRightValue_helper() const;
    QString mBottom;
    QString defaultBottomValue_helper() const;
    QString mLeft;
    QString defaultLeftValue_helper() const;

    // Effect-windowview
    QList<int> mTouchBorderActivateAll;
    QList<int> defaultTouchBorderActivateAllValue_helper() const;
    QList<int> mTouchBorderActivatePresentWindows;
    QList<int> defaultTouchBorderActivatePresentWindowsValue_helper() const;
    QList<int> mTouchBorderActivateClass;
    QList<int> defaultTouchBorderActivateClassValue_helper() const;

    // Effect-overview
    QList<int> mTouchBorderActivateOverview;
    QList<int> defaultTouchBorderActivateOverviewValue_helper() const;
    QList<int> mTouchBorderActivateGrid;
    QList<int> defaultTouchBorderActivateGridValue_helper() const;

    // TabBox
    QList<int> mTouchBorderActivateTabBox;
    QList<int> defaultTouchBorderActivateTabBoxValue_helper() const;
    QList<int> mTouchBorderAlternativeActivate;
    QList<int> defaultTouchBorderAlternativeActivateValue_helper() const;
};

}

#endif

