// This file is generated by kconfig_compiler_kf6 from kwin.kcfg.
// All changes you do to this file will be lost.

#include "settings.h"

using namespace KWin;

Settings::Settings( KSharedConfig::Ptr config )
  : KConfigSkeleton( std::move( config ) )
{
  setCurrentGroup( QStringLiteral( "MouseBindings" ) );

  KConfigSkeleton::ItemString  *itemCommandTitlebarWheel;
  itemCommandTitlebarWheel = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandTitlebarWheel" ), mCommandTitlebarWheel, QStringLiteral( "Nothing" ) );
  addItem( itemCommandTitlebarWheel, QStringLiteral( "CommandTitlebarWheel" ) );
  KConfigSkeleton::ItemString  *itemCommandAllKey;
  itemCommandAllKey = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandAllKey" ), mCommandAllKey, QStringLiteral( "Meta" ) );
  addItem( itemCommandAllKey, QStringLiteral( "CommandAllKey" ) );
  KConfigSkeleton::ItemString  *itemCommandAllWheel;
  itemCommandAllWheel = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandAllWheel" ), mCommandAllWheel, QStringLiteral( "Nothing" ) );
  addItem( itemCommandAllWheel, QStringLiteral( "CommandAllWheel" ) );
  KConfigSkeleton::ItemString  *itemCommandActiveTitlebar1;
  itemCommandActiveTitlebar1 = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandActiveTitlebar1" ), mCommandActiveTitlebar1, QStringLiteral( "Raise" ) );
  addItem( itemCommandActiveTitlebar1, QStringLiteral( "CommandActiveTitlebar1" ) );
  KConfigSkeleton::ItemString  *itemCommandActiveTitlebar2;
  itemCommandActiveTitlebar2 = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandActiveTitlebar2" ), mCommandActiveTitlebar2, QStringLiteral( "Nothing" ) );
  addItem( itemCommandActiveTitlebar2, QStringLiteral( "CommandActiveTitlebar2" ) );
  KConfigSkeleton::ItemString  *itemCommandActiveTitlebar3;
  itemCommandActiveTitlebar3 = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandActiveTitlebar3" ), mCommandActiveTitlebar3, QStringLiteral( "Operations menu" ) );
  addItem( itemCommandActiveTitlebar3, QStringLiteral( "CommandActiveTitlebar3" ) );
  KConfigSkeleton::ItemString  *itemCommandInactiveTitlebar1;
  itemCommandInactiveTitlebar1 = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandInactiveTitlebar1" ), mCommandInactiveTitlebar1, QStringLiteral( "Activate and raise" ) );
  addItem( itemCommandInactiveTitlebar1, QStringLiteral( "CommandInactiveTitlebar1" ) );
  KConfigSkeleton::ItemString  *itemCommandInactiveTitlebar2;
  itemCommandInactiveTitlebar2 = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandInactiveTitlebar2" ), mCommandInactiveTitlebar2, QStringLiteral( "Nothing" ) );
  addItem( itemCommandInactiveTitlebar2, QStringLiteral( "CommandInactiveTitlebar2" ) );
  KConfigSkeleton::ItemString  *itemCommandInactiveTitlebar3;
  itemCommandInactiveTitlebar3 = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandInactiveTitlebar3" ), mCommandInactiveTitlebar3, QStringLiteral( "Operations menu" ) );
  addItem( itemCommandInactiveTitlebar3, QStringLiteral( "CommandInactiveTitlebar3" ) );
  KConfigSkeleton::ItemString  *itemCommandWindow1;
  itemCommandWindow1 = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandWindow1" ), mCommandWindow1, QStringLiteral( "Activate, pass click and raise on release" ) );
  addItem( itemCommandWindow1, QStringLiteral( "CommandWindow1" ) );
  KConfigSkeleton::ItemString  *itemCommandWindow2;
  itemCommandWindow2 = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandWindow2" ), mCommandWindow2, QStringLiteral( "Activate and pass click" ) );
  addItem( itemCommandWindow2, QStringLiteral( "CommandWindow2" ) );
  KConfigSkeleton::ItemString  *itemCommandWindow3;
  itemCommandWindow3 = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandWindow3" ), mCommandWindow3, QStringLiteral( "Activate and pass click" ) );
  addItem( itemCommandWindow3, QStringLiteral( "CommandWindow3" ) );
  KConfigSkeleton::ItemString  *itemCommandWindowWheel;
  itemCommandWindowWheel = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandWindowWheel" ), mCommandWindowWheel, QStringLiteral( "Scroll" ) );
  addItem( itemCommandWindowWheel, QStringLiteral( "CommandWindowWheel" ) );
  KConfigSkeleton::ItemString  *itemCommandAll1;
  itemCommandAll1 = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandAll1" ), mCommandAll1, QStringLiteral( "Move" ) );
  addItem( itemCommandAll1, QStringLiteral( "CommandAll1" ) );
  KConfigSkeleton::ItemString  *itemCommandAll2;
  itemCommandAll2 = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandAll2" ), mCommandAll2, QStringLiteral( "Toggle raise and lower" ) );
  addItem( itemCommandAll2, QStringLiteral( "CommandAll2" ) );
  KConfigSkeleton::ItemString  *itemCommandAll3;
  itemCommandAll3 = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CommandAll3" ), mCommandAll3, QStringLiteral( "Resize" ) );
  addItem( itemCommandAll3, QStringLiteral( "CommandAll3" ) );
  KConfigSkeleton::ItemBool  *itemDoubleClickBorderToMaximize;
  itemDoubleClickBorderToMaximize = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "DoubleClickBorderToMaximize" ), mDoubleClickBorderToMaximize, true );
  addItem( itemDoubleClickBorderToMaximize, QStringLiteral( "DoubleClickBorderToMaximize" ) );

  setCurrentGroup( QStringLiteral( "ElectricBorders" ) );

  KConfigSkeleton::ItemString  *itemTop;
  itemTop = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Top" ), mTop, QStringLiteral( "None" ) );
  addItem( itemTop, QStringLiteral( "Top" ) );
  KConfigSkeleton::ItemString  *itemTopRight;
  itemTopRight = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "TopRight" ), mTopRight, QStringLiteral( "None" ) );
  addItem( itemTopRight, QStringLiteral( "TopRight" ) );
  KConfigSkeleton::ItemString  *itemRight;
  itemRight = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Right" ), mRight, QStringLiteral( "None" ) );
  addItem( itemRight, QStringLiteral( "Right" ) );
  KConfigSkeleton::ItemString  *itemBottomRight;
  itemBottomRight = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "BottomRight" ), mBottomRight, QStringLiteral( "None" ) );
  addItem( itemBottomRight, QStringLiteral( "BottomRight" ) );
  KConfigSkeleton::ItemString  *itemBottom;
  itemBottom = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Bottom" ), mBottom, QStringLiteral( "None" ) );
  addItem( itemBottom, QStringLiteral( "Bottom" ) );
  KConfigSkeleton::ItemString  *itemBottomLeft;
  itemBottomLeft = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "BottomLeft" ), mBottomLeft, QStringLiteral( "None" ) );
  addItem( itemBottomLeft, QStringLiteral( "BottomLeft" ) );
  KConfigSkeleton::ItemString  *itemLeft;
  itemLeft = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Left" ), mLeft, QStringLiteral( "None" ) );
  addItem( itemLeft, QStringLiteral( "Left" ) );
  KConfigSkeleton::ItemString  *itemTopLeft;
  itemTopLeft = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "TopLeft" ), mTopLeft, QStringLiteral( "None" ) );
  addItem( itemTopLeft, QStringLiteral( "TopLeft" ) );

  setCurrentGroup( QStringLiteral( "Windows" ) );

  KConfigSkeleton::ItemBool  *itemCondensedTitle;
  itemCondensedTitle = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "CondensedTitle" ), mCondensedTitle, false );
  addItem( itemCondensedTitle, QStringLiteral( "CondensedTitle" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesFocusPolicy;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ClickToFocus");
    valuesFocusPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("FocusFollowsMouse");
    valuesFocusPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("FocusUnderMouse");
    valuesFocusPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("FocusStrictlyUnderMouse");
    valuesFocusPolicy.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemFocusPolicy;
  itemFocusPolicy = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "FocusPolicy" ), mFocusPolicy, valuesFocusPolicy, Options::ClickToFocus );
  addItem( itemFocusPolicy, QStringLiteral( "FocusPolicy" ) );
  KConfigSkeleton::ItemBool  *itemNextFocusPrefersMouse;
  itemNextFocusPrefersMouse = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "NextFocusPrefersMouse" ), mNextFocusPrefersMouse, false );
  addItem( itemNextFocusPrefersMouse, QStringLiteral( "NextFocusPrefersMouse" ) );
  KConfigSkeleton::ItemBool  *itemSeparateScreenFocus;
  itemSeparateScreenFocus = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "SeparateScreenFocus" ), mSeparateScreenFocus, true );
  addItem( itemSeparateScreenFocus, QStringLiteral( "SeparateScreenFocus" ) );
  KConfigSkeleton::ItemBool  *itemRollOverDesktops;
  itemRollOverDesktops = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "RollOverDesktops" ), mRollOverDesktops, false );
  addItem( itemRollOverDesktops, QStringLiteral( "RollOverDesktops" ) );
  KConfigSkeleton::ItemBool  *itemPerOutputVirtualDesktops;
  itemPerOutputVirtualDesktops = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "PerOutputVirtualDesktops" ), mPerOutputVirtualDesktops, false );
  addItem( itemPerOutputVirtualDesktops, QStringLiteral( "PerOutputVirtualDesktops" ) );
  KConfigSkeleton::ItemInt  *itemFocusStealingPreventionLevel;
  itemFocusStealingPreventionLevel = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "FocusStealingPreventionLevel" ), mFocusStealingPreventionLevel, 1 );
  itemFocusStealingPreventionLevel->setMinValue(0);
  itemFocusStealingPreventionLevel->setMaxValue(4);
  addItem( itemFocusStealingPreventionLevel, QStringLiteral( "FocusStealingPreventionLevel" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesPlacement;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("PlacementNone");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("PlacementDefault");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("PlacementUnknown");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("PlacementRandom");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("PlacementSmart");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("PlacementCentered");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("PlacementZeroCornered");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("PlacementUnderMouse");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("PlacementOnMainWindow");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("PlacementMaximizing");
    valuesPlacement.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemPlacement;
  itemPlacement = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "Placement" ), mPlacement, valuesPlacement, [] {
                #if KWIN_BUILD_DECORATIONS
                    return PlacementCentered;
                #else
                    return PlacementMaximizing;
                #endif
            }() );
  itemPlacement->setValueForChoice(QStringLiteral( "PlacementNone" ), QStringLiteral( "NoPlacement" ));
  itemPlacement->setValueForChoice(QStringLiteral( "PlacementDefault" ), QStringLiteral( "Default" ));
  itemPlacement->setValueForChoice(QStringLiteral( "PlacementUnknown" ), QStringLiteral( "Unknown" ));
  itemPlacement->setValueForChoice(QStringLiteral( "PlacementRandom" ), QStringLiteral( "Random" ));
  itemPlacement->setValueForChoice(QStringLiteral( "PlacementSmart" ), QStringLiteral( "Smart" ));
  itemPlacement->setValueForChoice(QStringLiteral( "PlacementCentered" ), QStringLiteral( "Centered" ));
  itemPlacement->setValueForChoice(QStringLiteral( "PlacementZeroCornered" ), QStringLiteral( "ZeroCornered" ));
  itemPlacement->setValueForChoice(QStringLiteral( "PlacementUnderMouse" ), QStringLiteral( "UnderMouse" ));
  itemPlacement->setValueForChoice(QStringLiteral( "PlacementOnMainWindow" ), QStringLiteral( "OnMainWindow" ));
  itemPlacement->setValueForChoice(QStringLiteral( "PlacementMaximizing" ), QStringLiteral( "Maximizing" ));
  addItem( itemPlacement, QStringLiteral( "Placement" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesActivationDesktopPolicy;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("SwitchToOtherDesktop");
    valuesActivationDesktopPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("BringToCurrentDesktop");
    valuesActivationDesktopPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("DoNothing");
    valuesActivationDesktopPolicy.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemActivationDesktopPolicy;
  itemActivationDesktopPolicy = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "ActivationDesktopPolicy" ), mActivationDesktopPolicy, valuesActivationDesktopPolicy, KWin::Options::ActivationDesktopPolicy::SwitchToOtherDesktop );
  addItem( itemActivationDesktopPolicy, QStringLiteral( "ActivationDesktopPolicy" ) );
  KConfigSkeleton::ItemBool  *itemAutoRaise;
  itemAutoRaise = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "AutoRaise" ), mAutoRaise, false );
  addItem( itemAutoRaise, QStringLiteral( "AutoRaise" ) );
  KConfigSkeleton::ItemInt  *itemAutoRaiseInterval;
  itemAutoRaiseInterval = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "AutoRaiseInterval" ), mAutoRaiseInterval, 750 );
  addItem( itemAutoRaiseInterval, QStringLiteral( "AutoRaiseInterval" ) );
  KConfigSkeleton::ItemInt  *itemDelayFocusInterval;
  itemDelayFocusInterval = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "DelayFocusInterval" ), mDelayFocusInterval, 300 );
  addItem( itemDelayFocusInterval, QStringLiteral( "DelayFocusInterval" ) );
  KConfigSkeleton::ItemBool  *itemClickRaise;
  itemClickRaise = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ClickRaise" ), mClickRaise, true );
  addItem( itemClickRaise, QStringLiteral( "ClickRaise" ) );
  KConfigSkeleton::ItemInt  *itemBorderSnapZone;
  itemBorderSnapZone = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "BorderSnapZone" ), mBorderSnapZone, 10 );
  addItem( itemBorderSnapZone, QStringLiteral( "BorderSnapZone" ) );
  KConfigSkeleton::ItemInt  *itemWindowSnapZone;
  itemWindowSnapZone = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "WindowSnapZone" ), mWindowSnapZone, 10 );
  addItem( itemWindowSnapZone, QStringLiteral( "WindowSnapZone" ) );
  KConfigSkeleton::ItemInt  *itemCenterSnapZone;
  itemCenterSnapZone = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "CenterSnapZone" ), mCenterSnapZone, 0 );
  addItem( itemCenterSnapZone, QStringLiteral( "CenterSnapZone" ) );
  KConfigSkeleton::ItemBool  *itemSnapOnlyWhenOverlapping;
  itemSnapOnlyWhenOverlapping = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "SnapOnlyWhenOverlapping" ), mSnapOnlyWhenOverlapping, false );
  addItem( itemSnapOnlyWhenOverlapping, QStringLiteral( "SnapOnlyWhenOverlapping" ) );
  KConfigSkeleton::ItemInt  *itemElectricBorders;
  itemElectricBorders = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "ElectricBorders" ), mElectricBorders, 0 );
  addItem( itemElectricBorders, QStringLiteral( "ElectricBorders" ) );
  KConfigSkeleton::ItemInt  *itemElectricBorderDelay;
  itemElectricBorderDelay = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "ElectricBorderDelay" ), mElectricBorderDelay, 75 );
  addItem( itemElectricBorderDelay, QStringLiteral( "ElectricBorderDelay" ) );
  KConfigSkeleton::ItemInt  *itemElectricBorderCooldown;
  itemElectricBorderCooldown = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "ElectricBorderCooldown" ), mElectricBorderCooldown, 350 );
  addItem( itemElectricBorderCooldown, QStringLiteral( "ElectricBorderCooldown" ) );
  KConfigSkeleton::ItemInt  *itemElectricBorderPushbackPixels;
  itemElectricBorderPushbackPixels = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "ElectricBorderPushbackPixels" ), mElectricBorderPushbackPixels, 1 );
  addItem( itemElectricBorderPushbackPixels, QStringLiteral( "ElectricBorderPushbackPixels" ) );
  KConfigSkeleton::ItemBool  *itemElectricBorderMaximize;
  itemElectricBorderMaximize = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ElectricBorderMaximize" ), mElectricBorderMaximize, true );
  addItem( itemElectricBorderMaximize, QStringLiteral( "ElectricBorderMaximize" ) );
  KConfigSkeleton::ItemBool  *itemElectricBorderAllScreenCorner;
  itemElectricBorderAllScreenCorner = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ElectricBorderAllScreenCorner" ), mElectricBorderAllScreenCorner, true );
  addItem( itemElectricBorderAllScreenCorner, QStringLiteral( "ElectricBorderAllScreenCorner" ) );
  KConfigSkeleton::ItemBool  *itemElectricBorderTiling;
  itemElectricBorderTiling = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ElectricBorderTiling" ), mElectricBorderTiling, true );
  addItem( itemElectricBorderTiling, QStringLiteral( "ElectricBorderTiling" ) );
  KConfigSkeleton::ItemDouble  *itemElectricBorderCornerRatio;
  itemElectricBorderCornerRatio = new KConfigSkeleton::ItemDouble( currentGroup(), QStringLiteral( "ElectricBorderCornerRatio" ), mElectricBorderCornerRatio, 0.25 );
  itemElectricBorderCornerRatio->setMinValue(0.0);
  itemElectricBorderCornerRatio->setMaxValue(1.0);
  addItem( itemElectricBorderCornerRatio, QStringLiteral( "ElectricBorderCornerRatio" ) );
  KConfigSkeleton::ItemString  *itemTitlebarDoubleClickCommand;
  itemTitlebarDoubleClickCommand = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "TitlebarDoubleClickCommand" ), mTitlebarDoubleClickCommand, QStringLiteral( "Maximize" ) );
  addItem( itemTitlebarDoubleClickCommand, QStringLiteral( "TitlebarDoubleClickCommand" ) );
  KConfigSkeleton::ItemString  *itemMaximizeButtonLeftClickCommand;
  itemMaximizeButtonLeftClickCommand = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "MaximizeButtonLeftClickCommand" ), mMaximizeButtonLeftClickCommand, QStringLiteral( "Maximize" ) );
  addItem( itemMaximizeButtonLeftClickCommand, QStringLiteral( "MaximizeButtonLeftClickCommand" ) );
  KConfigSkeleton::ItemString  *itemMaximizeButtonMiddleClickCommand;
  itemMaximizeButtonMiddleClickCommand = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "MaximizeButtonMiddleClickCommand" ), mMaximizeButtonMiddleClickCommand, QStringLiteral( "Maximize (vertical only)" ) );
  addItem( itemMaximizeButtonMiddleClickCommand, QStringLiteral( "MaximizeButtonMiddleClickCommand" ) );
  KConfigSkeleton::ItemString  *itemMaximizeButtonRightClickCommand;
  itemMaximizeButtonRightClickCommand = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "MaximizeButtonRightClickCommand" ), mMaximizeButtonRightClickCommand, QStringLiteral( "Maximize (horizontal only)" ) );
  addItem( itemMaximizeButtonRightClickCommand, QStringLiteral( "MaximizeButtonRightClickCommand" ) );
  KConfigSkeleton::ItemInt  *itemKillPingTimeout;
  itemKillPingTimeout = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "KillPingTimeout" ), mKillPingTimeout, 5000 );
  addItem( itemKillPingTimeout, QStringLiteral( "KillPingTimeout" ) );
  KConfigSkeleton::ItemBool  *itemInactiveTabsSkipTaskbar;
  itemInactiveTabsSkipTaskbar = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "InactiveTabsSkipTaskbar" ), mInactiveTabsSkipTaskbar, false );
  addItem( itemInactiveTabsSkipTaskbar, QStringLiteral( "InactiveTabsSkipTaskbar" ) );
  KConfigSkeleton::ItemBool  *itemAutogroupSimilarWindows;
  itemAutogroupSimilarWindows = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "AutogroupSimilarWindows" ), mAutogroupSimilarWindows, false );
  addItem( itemAutogroupSimilarWindows, QStringLiteral( "AutogroupSimilarWindows" ) );
  KConfigSkeleton::ItemBool  *itemAutogroupInForeground;
  itemAutogroupInForeground = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "AutogroupInForeground" ), mAutogroupInForeground, true );
  addItem( itemAutogroupInForeground, QStringLiteral( "AutogroupInForeground" ) );
  KConfigSkeleton::ItemBool  *itemBorderlessMaximizedWindows;
  itemBorderlessMaximizedWindows = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "BorderlessMaximizedWindows" ), mBorderlessMaximizedWindows, false );
  addItem( itemBorderlessMaximizedWindows, QStringLiteral( "BorderlessMaximizedWindows" ) );
  KConfigSkeleton::ItemBool  *itemInteractiveWindowMoveEnabled;
  itemInteractiveWindowMoveEnabled = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "InteractiveWindowMoveEnabled" ), mInteractiveWindowMoveEnabled, true );
  addItem( itemInteractiveWindowMoveEnabled, QStringLiteral( "InteractiveWindowMoveEnabled" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesPictureInPictureHomeCorner;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("TopLeftCorner");
    valuesPictureInPictureHomeCorner.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("TopRightCorner");
    valuesPictureInPictureHomeCorner.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("BottomLeftCorner");
    valuesPictureInPictureHomeCorner.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("BottomRightCorner");
    valuesPictureInPictureHomeCorner.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemPictureInPictureHomeCorner;
  itemPictureInPictureHomeCorner = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "PictureInPictureHomeCorner" ), mPictureInPictureHomeCorner, valuesPictureInPictureHomeCorner, Qt::BottomRightCorner );
  addItem( itemPictureInPictureHomeCorner, QStringLiteral( "PictureInPictureHomeCorner" ) );
  KConfigSkeleton::ItemInt  *itemPictureInPictureMargin;
  itemPictureInPictureMargin = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "PictureInPictureMargin" ), mPictureInPictureMargin, 20 );
  addItem( itemPictureInPictureMargin, QStringLiteral( "PictureInPictureMargin" ) );
  KConfigSkeleton::ItemBool  *itemOverlayVirtualKeyboardOnWindows;
  itemOverlayVirtualKeyboardOnWindows = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "OverlayVirtualKeyboardOnWindows" ), mOverlayVirtualKeyboardOnWindows, false );
  addItem( itemOverlayVirtualKeyboardOnWindows, QStringLiteral( "OverlayVirtualKeyboardOnWindows" ) );

  setCurrentGroup( QStringLiteral( "EdgeBarrier" ) );

  KConfigSkeleton::ItemBool  *itemCornerBarrier;
  itemCornerBarrier = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "CornerBarrier" ), mCornerBarrier, true );
  addItem( itemCornerBarrier, QStringLiteral( "CornerBarrier" ) );
  KConfigSkeleton::ItemInt  *itemEdgeBarrier;
  itemEdgeBarrier = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "EdgeBarrier" ), mEdgeBarrier, 100 );
  itemEdgeBarrier->setMinValue(0);
  itemEdgeBarrier->setMaxValue(1000);
  addItem( itemEdgeBarrier, QStringLiteral( "EdgeBarrier" ) );

  setCurrentGroup( QStringLiteral( "ScreenEdges" ) );

  KConfigSkeleton::ItemBool  *itemRemainActiveOnFullscreen;
  itemRemainActiveOnFullscreen = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "RemainActiveOnFullscreen" ), mRemainActiveOnFullscreen, false );
  addItem( itemRemainActiveOnFullscreen, QStringLiteral( "RemainActiveOnFullscreen" ) );
  KConfigSkeleton::ItemInt  *itemTouchTarget;
  itemTouchTarget = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "TouchTarget" ), mTouchTarget, 8 );
  itemTouchTarget->setMinValue(1);
  addItem( itemTouchTarget, QStringLiteral( "TouchTarget" ) );

  setCurrentGroup( QStringLiteral( "Compositing" ) );

  KConfigSkeleton::ItemString  *itemBackend;
  itemBackend = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Backend" ), mBackend, QStringLiteral( "OpenGL" ) );
  addItem( itemBackend, QStringLiteral( "Backend" ) );
  KConfigSkeleton::ItemBool  *itemEnabled;
  itemEnabled = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Enabled" ), mEnabled, true );
  addItem( itemEnabled, QStringLiteral( "Enabled" ) );
  KConfigSkeleton::ItemInt  *itemLastFailureTimestamp;
  itemLastFailureTimestamp = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "LastFailureTimestamp" ), mLastFailureTimestamp, 0 );
  addItem( itemLastFailureTimestamp, QStringLiteral( "LastFailureTimestamp" ) );
  KConfigSkeleton::ItemBool  *itemAllowTearing;
  itemAllowTearing = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "AllowTearing" ), mAllowTearing, true );
  addItem( itemAllowTearing, QStringLiteral( "AllowTearing" ) );

  setCurrentGroup( QStringLiteral( "TabBox" ) );

  KConfigSkeleton::ItemInt  *itemDelayTime;
  itemDelayTime = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "DelayTime" ), mDelayTime, 90 );
  addItem( itemDelayTime, QStringLiteral( "DelayTime" ) );
  KConfigSkeleton::ItemUInt  *itemDesktopMode;
  itemDesktopMode = new KConfigSkeleton::ItemUInt( currentGroup(), QStringLiteral( "DesktopMode" ), mDesktopMode, 1 );
  addItem( itemDesktopMode, QStringLiteral( "DesktopMode" ) );
  KConfigSkeleton::ItemUInt  *itemActivitiesMode;
  itemActivitiesMode = new KConfigSkeleton::ItemUInt( currentGroup(), QStringLiteral( "ActivitiesMode" ), mActivitiesMode, 1 );
  addItem( itemActivitiesMode, QStringLiteral( "ActivitiesMode" ) );
  KConfigSkeleton::ItemUInt  *itemApplicationsMode;
  itemApplicationsMode = new KConfigSkeleton::ItemUInt( currentGroup(), QStringLiteral( "ApplicationsMode" ), mApplicationsMode, 0 );
  addItem( itemApplicationsMode, QStringLiteral( "ApplicationsMode" ) );
  KConfigSkeleton::ItemUInt  *itemMinimizedMode;
  itemMinimizedMode = new KConfigSkeleton::ItemUInt( currentGroup(), QStringLiteral( "MinimizedMode" ), mMinimizedMode, 0 );
  addItem( itemMinimizedMode, QStringLiteral( "MinimizedMode" ) );
  KConfigSkeleton::ItemUInt  *itemShowDesktopMode;
  itemShowDesktopMode = new KConfigSkeleton::ItemUInt( currentGroup(), QStringLiteral( "ShowDesktopMode" ), mShowDesktopMode, 0 );
  addItem( itemShowDesktopMode, QStringLiteral( "ShowDesktopMode" ) );
  KConfigSkeleton::ItemUInt  *itemMultiScreenMode;
  itemMultiScreenMode = new KConfigSkeleton::ItemUInt( currentGroup(), QStringLiteral( "MultiScreenMode" ), mMultiScreenMode, 0 );
  addItem( itemMultiScreenMode, QStringLiteral( "MultiScreenMode" ) );
  KConfigSkeleton::ItemUInt  *itemSwitchingMode;
  itemSwitchingMode = new KConfigSkeleton::ItemUInt( currentGroup(), QStringLiteral( "SwitchingMode" ), mSwitchingMode, 0 );
  addItem( itemSwitchingMode, QStringLiteral( "SwitchingMode" ) );
  KConfigSkeleton::ItemBool  *itemShowOutline;
  itemShowOutline = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowOutline" ), mShowOutline, true );
  addItem( itemShowOutline, QStringLiteral( "ShowOutline" ) );
  KConfigSkeleton::ItemBool  *itemShowTabBox;
  itemShowTabBox = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowTabBox" ), mShowTabBox, true );
  addItem( itemShowTabBox, QStringLiteral( "ShowTabBox" ) );
  KConfigSkeleton::ItemBool  *itemHighlightWindows;
  itemHighlightWindows = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "HighlightWindows" ), mHighlightWindows, true );
  addItem( itemHighlightWindows, QStringLiteral( "HighlightWindows" ) );
  KConfigSkeleton::ItemString  *itemLayoutName;
  itemLayoutName = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "LayoutName" ), mLayoutName, QStringLiteral( "thumbnail_grid" ) );
  addItem( itemLayoutName, QStringLiteral( "LayoutName" ) );

  setCurrentGroup( QStringLiteral( "KDE" ) );

  KConfigSkeleton::ItemDouble  *itemAnimationDurationFactor;
  itemAnimationDurationFactor = new KConfigSkeleton::ItemDouble( currentGroup(), QStringLiteral( "AnimationDurationFactor" ), mAnimationDurationFactor, 1 );
  itemAnimationDurationFactor->setMinValue(0);
  addItem( itemAnimationDurationFactor, QStringLiteral( "AnimationDurationFactor" ) );

  setCurrentGroup( QStringLiteral( "Wayland" ) );

  KConfigSkeleton::ItemPath  *itemInputMethod;
  itemInputMethod = new KConfigSkeleton::ItemPath( currentGroup(), QStringLiteral( "InputMethod" ), mInputMethod );
  addItem( itemInputMethod, QStringLiteral( "InputMethod" ) );
  KConfigSkeleton::ItemBool  *itemDoubleTapWakeup;
  itemDoubleTapWakeup = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "DoubleTapWakeup" ), mDoubleTapWakeup, true );
  addItem( itemDoubleTapWakeup, QStringLiteral( "DoubleTapWakeup" ) );
  KConfigSkeleton::ItemBool  *itemEnablePrimarySelection;
  itemEnablePrimarySelection = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "EnablePrimarySelection" ), mEnablePrimarySelection, true );
  addItem( itemEnablePrimarySelection, QStringLiteral( "EnablePrimarySelection" ) );

  setCurrentGroup( QStringLiteral( "Xwayland" ) );

  QList<KConfigSkeleton::ItemEnum::Choice> valuesXwaylandCrashPolicy;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Stop");
    valuesXwaylandCrashPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Restart");
    valuesXwaylandCrashPolicy.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemXwaylandCrashPolicy;
  itemXwaylandCrashPolicy = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "XwaylandCrashPolicy" ), mXwaylandCrashPolicy, valuesXwaylandCrashPolicy, XwaylandCrashPolicy::Restart );
  addItem( itemXwaylandCrashPolicy, QStringLiteral( "XwaylandCrashPolicy" ) );
  KConfigSkeleton::ItemUInt  *itemXwaylandMaxCrashCount;
  itemXwaylandMaxCrashCount = new KConfigSkeleton::ItemUInt( currentGroup(), QStringLiteral( "XwaylandMaxCrashCount" ), mXwaylandMaxCrashCount, 3 );
  addItem( itemXwaylandMaxCrashCount, QStringLiteral( "XwaylandMaxCrashCount" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesXwaylandEavesdrops;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("None");
    valuesXwaylandEavesdrops.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Modifiers");
    valuesXwaylandEavesdrops.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Combinations");
    valuesXwaylandEavesdrops.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("All");
    valuesXwaylandEavesdrops.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemXwaylandEavesdrops;
  itemXwaylandEavesdrops = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "XwaylandEavesdrops" ), mXwaylandEavesdrops, valuesXwaylandEavesdrops, EnumXwaylandEavesdrops::Combinations );
  addItem( itemXwaylandEavesdrops, QStringLiteral( "XwaylandEavesdrops" ) );
  KConfigSkeleton::ItemBool  *itemXwaylandEavesdropsMouse;
  itemXwaylandEavesdropsMouse = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "XwaylandEavesdropsMouse" ), mXwaylandEavesdropsMouse, false );
  addItem( itemXwaylandEavesdropsMouse, QStringLiteral( "XwaylandEavesdropsMouse" ) );
  KConfigSkeleton::ItemBool  *itemXwaylandEisNoPrompt;
  itemXwaylandEisNoPrompt = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "XwaylandEisNoPrompt" ), mXwaylandEisNoPrompt, false );
  addItem( itemXwaylandEisNoPrompt, QStringLiteral( "XwaylandEisNoPrompt" ) );
  KConfigSkeleton::ItemStringList  *itemXwaylandEisNoPromptApps;
  itemXwaylandEisNoPromptApps = new KConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "XwaylandEisNoPromptApps" ), mXwaylandEisNoPromptApps );
  addItem( itemXwaylandEisNoPromptApps, QStringLiteral( "XwaylandEisNoPromptApps" ) );
}

Settings::~Settings()
{
}

