/*
    SPDX-FileCopyrightText: 2022 Vlad Zahorodnii <vlad.zahorodnii@kde.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#pragma once

#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(KWIN_VIRTUAL)
