// This file is autogenerated. Changes will be overwritten.
#include "EWIEGA46WW/moc_eisbackend.cpp"
#include "EWIEGA46WW/moc_eisdevice.cpp"
#include "EWIEGA46WW/moc_eisinputcapture.cpp"
#include "EWIEGA46WW/moc_eisinputcapturemanager.cpp"
#include "EWIEGA46WW/moc_eisplugin.cpp"
