// This file is generated by kconfig_compiler_kf6 from kscreen.kcfg.
// All changes you do to this file will be lost.
#ifndef KWIN_KSCREENCONFIG_H
#define KWIN_KSCREENCONFIG_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

namespace KWin {

class KscreenConfig : public KConfigSkeleton
{
  public:

    static KscreenConfig *self();
    static void instance(const QString& cfgfilename);
    static void instance(KSharedConfig::Ptr config);
    ~KscreenConfig() override;

    /**
      Set Duration
    */
    static
    void setDuration( int v )
    {
      if (!self()->isDurationImmutable())
        self()->mDuration = v;
    }

    /**
      Get Duration
    */
    static
    int duration()
    {
      return self()->mDuration;
    }

    /**
      Is Duration Immutable
    */
    static
    bool isDurationImmutable()
    {
      return self()->isImmutable( QStringLiteral( "Duration" ) );
    }

  protected:
    KscreenConfig(KSharedConfig::Ptr config);
    friend class KscreenConfigHelper;


    // Effect-kscreen
    int mDuration;
};

}

#endif

