/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kwin.Effects.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef EFFECTSADAPTOR_H
#define EFFECTSADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "effect/effecthandler.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.kde.kwin.Effects
 */
class EffectsAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.kwin.Effects")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.kde.kwin.Effects\">\n"
"    <property access=\"read\" type=\"as\" name=\"activeEffects\"/>\n"
"    <property access=\"read\" type=\"as\" name=\"loadedEffects\"/>\n"
"    <property access=\"read\" type=\"as\" name=\"listOfEffects\"/>\n"
"    <method name=\"reconfigureEffect\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"    <method name=\"loadEffect\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"    <method name=\"toggleEffect\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"    <method name=\"unloadEffect\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"    <method name=\"isEffectLoaded\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"    <method name=\"isEffectSupported\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"    <method name=\"areEffectsSupported\">\n"
"      <arg direction=\"out\" type=\"ab\"/>\n"
"      <annotation value=\"QList&lt;bool&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"      <arg direction=\"in\" type=\"as\" name=\"names\"/>\n"
"    </method>\n"
"    <method name=\"supportInformation\">\n"
"      <arg direction=\"out\" type=\"s\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"    <method name=\"debug\">\n"
"      <arg direction=\"out\" type=\"s\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    EffectsAdaptor(KWin::EffectsHandler *parent);
    ~EffectsAdaptor() override;

    inline KWin::EffectsHandler *parent() const
    { return static_cast<KWin::EffectsHandler *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(QStringList activeEffects READ activeEffects)
    QStringList activeEffects() const;

    Q_PROPERTY(QStringList listOfEffects READ listOfEffects)
    QStringList listOfEffects() const;

    Q_PROPERTY(QStringList loadedEffects READ loadedEffects)
    QStringList loadedEffects() const;

public Q_SLOTS: // METHODS
    QList<bool> areEffectsSupported(const QStringList &names);
    QString debug(const QString &name, const QString &name_);
    bool isEffectLoaded(const QString &name);
    bool isEffectSupported(const QString &name);
    bool loadEffect(const QString &name);
    void reconfigureEffect(const QString &name);
    QString supportInformation(const QString &name);
    void toggleEffect(const QString &name);
    void unloadEffect(const QString &name);
Q_SIGNALS: // SIGNALS
};

#endif
