/* Generated by wayland-scanner 1.26.0 */

/*
 * SPDX-FileCopyrightText: 2018 Marco Martin
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface org_kde_plasma_virtual_desktop_interface;

static const struct wl_interface *org_kde_plasma_virtual_desktop_types[] = {
	NULL,
	NULL,
	&org_kde_plasma_virtual_desktop_interface,
	NULL,
};

static const struct wl_message org_kde_plasma_virtual_desktop_management_requests[] = {
	{ "get_virtual_desktop", "ns", org_kde_plasma_virtual_desktop_types + 2 },
	{ "request_create_virtual_desktop", "su", org_kde_plasma_virtual_desktop_types + 0 },
	{ "request_remove_virtual_desktop", "s", org_kde_plasma_virtual_desktop_types + 0 },
};

static const struct wl_message org_kde_plasma_virtual_desktop_management_events[] = {
	{ "desktop_created", "su", org_kde_plasma_virtual_desktop_types + 0 },
	{ "desktop_removed", "s", org_kde_plasma_virtual_desktop_types + 0 },
	{ "done", "", org_kde_plasma_virtual_desktop_types + 0 },
	{ "rows", "2u", org_kde_plasma_virtual_desktop_types + 0 },
};

WL_PRIVATE const struct wl_interface org_kde_plasma_virtual_desktop_management_interface = {
	"org_kde_plasma_virtual_desktop_management", 4,
	3, org_kde_plasma_virtual_desktop_management_requests,
	4, org_kde_plasma_virtual_desktop_management_events,
};

static const struct wl_message org_kde_plasma_virtual_desktop_requests[] = {
	{ "request_activate", "", org_kde_plasma_virtual_desktop_types + 0 },
	{ "request_enter_output", "4s", org_kde_plasma_virtual_desktop_types + 0 },
};

static const struct wl_message org_kde_plasma_virtual_desktop_events[] = {
	{ "desktop_id", "s", org_kde_plasma_virtual_desktop_types + 0 },
	{ "name", "s", org_kde_plasma_virtual_desktop_types + 0 },
	{ "activated", "", org_kde_plasma_virtual_desktop_types + 0 },
	{ "deactivated", "", org_kde_plasma_virtual_desktop_types + 0 },
	{ "done", "", org_kde_plasma_virtual_desktop_types + 0 },
	{ "removed", "", org_kde_plasma_virtual_desktop_types + 0 },
	{ "position", "3u", org_kde_plasma_virtual_desktop_types + 0 },
	{ "output_entered", "4s", org_kde_plasma_virtual_desktop_types + 0 },
};

WL_PRIVATE const struct wl_interface org_kde_plasma_virtual_desktop_interface = {
	"org_kde_plasma_virtual_desktop", 4,
	2, org_kde_plasma_virtual_desktop_requests,
	8, org_kde_plasma_virtual_desktop_events,
};

