// This file is generated by kconfig_compiler_kf6 from virtualkeyboardsettings.kcfg.
// All changes you do to this file will be lost.

#include "virtualkeyboardsettings.h"

VirtualKeyboardSettings::VirtualKeyboardSettings( QObject *parent )
  : KConfigSkeleton( QStringLiteral( "kwinrc" ) )
{
  setParent(parent);
  KConfigCompilerSignallingItem::NotifyFunction notifyFunction = static_cast<KConfigCompilerSignallingItem::NotifyFunction>(&VirtualKeyboardSettings::itemChanged);

  setCurrentGroup( QStringLiteral( "Wayland" ) );

  KConfigCompilerSignallingItem  *itemInputMethod;
  KConfigSkeleton::ItemPath  *innerItemInputMethod;
  innerItemInputMethod = new KConfigSkeleton::ItemPath( currentGroup(), QStringLiteral( "InputMethod" ), mInputMethod );
  itemInputMethod = new KConfigCompilerSignallingItem(innerItemInputMethod, this, notifyFunction, signalInputMethodChanged);
  itemInputMethod->setWriteFlags(KConfigBase::Notify);
  addItem( itemInputMethod, QStringLiteral( "InputMethod" ) );
}

VirtualKeyboardSettings::~VirtualKeyboardSettings()
{
}


void VirtualKeyboardSettings::itemChanged(quint64 signalFlag) {

  switch (signalFlag) {
  case signalInputMethodChanged:
    Q_EMIT InputMethodChanged();
    break;
  }
}
