/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.KWin.VirtualDesktopManager.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef VIRTUALDESKTOPMANAGERADAPTOR_H
#define VIRTUALDESKTOPMANAGERADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "dbusinterface.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.kde.KWin.VirtualDesktopManager
 */
class VirtualDesktopManagerAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.KWin.VirtualDesktopManager")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.kde.KWin.VirtualDesktopManager\">\n"
"    <property access=\"read\" type=\"u\" name=\"count\"/>\n"
"    <property access=\"readwrite\" type=\"s\" name=\"current\"/>\n"
"    <property access=\"readwrite\" type=\"u\" name=\"rows\"/>\n"
"    <property access=\"readwrite\" type=\"b\" name=\"navigationWrappingAround\"/>\n"
"    <property access=\"read\" type=\"a(iss)\" name=\"desktops\">\n"
"      <annotation value=\"KWin::DBusDesktopDataVector\" name=\"org.qtproject.QtDBus.QtTypeName\"/>\n"
"    </property>\n"
"    <signal name=\"countChanged\">\n"
"      <arg direction=\"out\" type=\"u\" name=\"count\"/>\n"
"    </signal>\n"
"    <signal name=\"rowsChanged\">\n"
"      <arg direction=\"out\" type=\"u\" name=\"rows\"/>\n"
"    </signal>\n"
"    <signal name=\"currentChanged\">\n"
"      <arg direction=\"out\" type=\"s\" name=\"id\"/>\n"
"    </signal>\n"
"    <signal name=\"navigationWrappingAroundChanged\">\n"
"      <arg direction=\"out\" type=\"b\" name=\"navigationWrappingAround\"/>\n"
"    </signal>\n"
"    <signal name=\"desktopDataChanged\">\n"
"      <arg direction=\"out\" type=\"s\" name=\"id\"/>\n"
"      <annotation value=\"KWin::DBusDesktopDataStruct\" name=\"org.qtproject.QtDBus.QtTypeName.Out1\"/>\n"
"      <arg direction=\"out\" type=\"(iss)\" name=\"desktopData\"/>\n"
"    </signal>\n"
"    <signal name=\"desktopCreated\">\n"
"      <arg direction=\"out\" type=\"s\" name=\"id\"/>\n"
"      <annotation value=\"KWin::DBusDesktopDataStruct\" name=\"org.qtproject.QtDBus.QtTypeName.Out1\"/>\n"
"      <arg direction=\"out\" type=\"(iss)\" name=\"desktopData\"/>\n"
"    </signal>\n"
"    <signal name=\"desktopRemoved\">\n"
"      <arg direction=\"out\" type=\"s\" name=\"id\"/>\n"
"    </signal>\n"
"    <method name=\"createDesktop\">\n"
"      <arg direction=\"in\" type=\"u\" name=\"position\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"    <method name=\"setDesktopName\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"id\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"    <method name=\"removeDesktop\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"id\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    VirtualDesktopManagerAdaptor(KWin::VirtualDesktopManagerDBusInterface *parent);
    ~VirtualDesktopManagerAdaptor() override;

    inline KWin::VirtualDesktopManagerDBusInterface *parent() const
    { return static_cast<KWin::VirtualDesktopManagerDBusInterface *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(uint count READ count)
    uint count() const;

    Q_PROPERTY(QString current READ current WRITE setCurrent)
    QString current() const;
    void setCurrent(const QString &value);

    Q_PROPERTY(KWin::DBusDesktopDataVector desktops READ desktops)
    KWin::DBusDesktopDataVector desktops() const;

    Q_PROPERTY(bool navigationWrappingAround READ navigationWrappingAround WRITE setNavigationWrappingAround)
    bool navigationWrappingAround() const;
    void setNavigationWrappingAround(bool value);

    Q_PROPERTY(uint rows READ rows WRITE setRows)
    uint rows() const;
    void setRows(uint value);

public Q_SLOTS: // METHODS
    void createDesktop(uint position, const QString &name);
    void removeDesktop(const QString &id);
    void setDesktopName(const QString &id, const QString &name);
Q_SIGNALS: // SIGNALS
    void countChanged(uint count);
    void currentChanged(const QString &id);
    void desktopCreated(const QString &id, KWin::DBusDesktopDataStruct desktopData);
    void desktopDataChanged(const QString &id, KWin::DBusDesktopDataStruct desktopData);
    void desktopRemoved(const QString &id);
    void navigationWrappingAroundChanged(bool navigationWrappingAround);
    void rowsChanged(uint rows);
};

#endif
