// This file was generated by kcmutils_generate_module_data(): DO NOT EDIT!

#ifndef KWINOPTIONSDATA_H
#define KWINOPTIONSDATA_H

#include <QObject>

#include "kcmoduledata.h"



class KWinOptionsSettings;
class KWinOptionsKDEGlobalsSettings;


class KWinOptionsData : public KCModuleData
{
    Q_OBJECT

public:
    explicit KWinOptionsData(QObject *parent = nullptr);
    KWinOptionsSettings *kWinOptionsSettings() const;
KWinOptionsKDEGlobalsSettings *kWinOptionsKDEGlobalsSettings() const;


private:
    KWinOptionsSettings *m_kWinOptionsSettings;
KWinOptionsKDEGlobalsSettings *m_kWinOptionsKDEGlobalsSettings;

};



#endif
