// This file was generated by kcmutils_generate_module_data(): DO NOT EDIT!

#include "kwinoptionsdata.h"

#include "kwinoptions_settings.h"
#include "kwinoptions_kdeglobals_settings.h"




KWinOptionsData::KWinOptionsData(QObject *parent)
: KCModuleData(parent)
, m_kWinOptionsSettings(new KWinOptionsSettings(this))
, m_kWinOptionsKDEGlobalsSettings(new KWinOptionsKDEGlobalsSettings(this))

{
autoRegisterSkeletons();
}

KWinOptionsSettings *KWinOptionsData::kWinOptionsSettings() const
{
    return m_kWinOptionsSettings;
}

KWinOptionsKDEGlobalsSettings *KWinOptionsData::kWinOptionsKDEGlobalsSettings() const
{
    return m_kWinOptionsKDEGlobalsSettings;
}





#include "moc_kwinoptionsdata.cpp"
