/* Generated by wayland-scanner 1.26.0 */

/*
 * SPDX-FileCopyrightText: 2024 Xaver Hugl <xaver.hugl@gmail.com>
 *
 * SPDX-License-Identifier: MIT-CMU
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface kde_external_brightness_device_v1_interface;

static const struct wl_interface *kde_external_brightness_v1_types[] = {
	NULL,
	&kde_external_brightness_device_v1_interface,
};

static const struct wl_message kde_external_brightness_v1_requests[] = {
	{ "destroy", "", kde_external_brightness_v1_types + 0 },
	{ "create_brightness_control", "n", kde_external_brightness_v1_types + 1 },
};

WL_PRIVATE const struct wl_interface kde_external_brightness_v1_interface = {
	"kde_external_brightness_v1", 3,
	2, kde_external_brightness_v1_requests,
	0, NULL,
};

static const struct wl_message kde_external_brightness_device_v1_requests[] = {
	{ "destroy", "", kde_external_brightness_v1_types + 0 },
	{ "set_internal", "u", kde_external_brightness_v1_types + 0 },
	{ "set_edid", "s", kde_external_brightness_v1_types + 0 },
	{ "set_max_brightness", "u", kde_external_brightness_v1_types + 0 },
	{ "commit", "", kde_external_brightness_v1_types + 0 },
	{ "set_observed_brightness", "2u", kde_external_brightness_v1_types + 0 },
	{ "set_uses_ddc_ci", "3u", kde_external_brightness_v1_types + 0 },
};

static const struct wl_message kde_external_brightness_device_v1_events[] = {
	{ "requested_brightness", "u", kde_external_brightness_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface kde_external_brightness_device_v1_interface = {
	"kde_external_brightness_device_v1", 3,
	7, kde_external_brightness_device_v1_requests,
	1, kde_external_brightness_device_v1_events,
};

