/*
    SPDX-FileCopyrightText: 2021 Vlad Zahorodnii <vlad.zahorodnii@kde.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "screencastsource.h"

namespace KWin
{

ScreenCastSource::ScreenCastSource()
{
}

} // namespace KWin

#include "moc_screencastsource.cpp"
