// This file is generated by kconfig_compiler_kf6 from kwinoptions_kdeglobals_settings.kcfg.
// All changes you do to this file will be lost.
#ifndef KWINOPTIONSKDEGLOBALSSETTINGS_H
#define KWINOPTIONSKDEGLOBALSSETTINGS_H

#include <options.h>

#include <qglobal.h>
#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class KWinOptionsKDEGlobalsSettings : public KConfigSkeleton
{
  public:

    KWinOptionsKDEGlobalsSettings( QObject *parent = nullptr );
    ~KWinOptionsKDEGlobalsSettings() override;

    /**
      Set AllowKDEAppsToRememberWindowPositions
    */
    void setAllowKDEAppsToRememberWindowPositions( bool v )
    {
      if (!isAllowKDEAppsToRememberWindowPositionsImmutable())
        mAllowKDEAppsToRememberWindowPositions = v;
    }

    /**
      Get AllowKDEAppsToRememberWindowPositions
    */
    bool allowKDEAppsToRememberWindowPositions() const
    {
      return mAllowKDEAppsToRememberWindowPositions;
    }

    /**
      Is AllowKDEAppsToRememberWindowPositions Immutable
    */
    bool isAllowKDEAppsToRememberWindowPositionsImmutable() const
    {
      return isImmutable( QStringLiteral( "AllowKDEAppsToRememberWindowPositions" ) );
    }

    /**
      Get AllowKDEAppsToRememberWindowPositions default value
    */
    bool defaultAllowKDEAppsToRememberWindowPositionsValue() const
    {
        return defaultAllowKDEAppsToRememberWindowPositionsValue_helper();
    }

  protected:

    // General
    bool mAllowKDEAppsToRememberWindowPositions;
    bool defaultAllowKDEAppsToRememberWindowPositionsValue_helper() const;
};

#endif

