/* Generated by wayland-scanner 1.26.0 */

/*
 * SPDX-FileCopyrightText: 2015 Martin Gräßlin
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface org_kde_kwin_shadow_interface;
extern const struct wl_interface wl_buffer_interface;
extern const struct wl_interface wl_surface_interface;

static const struct wl_interface *shadow_types[] = {
	NULL,
	&org_kde_kwin_shadow_interface,
	&wl_surface_interface,
	&wl_surface_interface,
	&wl_buffer_interface,
	&wl_buffer_interface,
	&wl_buffer_interface,
	&wl_buffer_interface,
	&wl_buffer_interface,
	&wl_buffer_interface,
	&wl_buffer_interface,
	&wl_buffer_interface,
};

static const struct wl_message org_kde_kwin_shadow_manager_requests[] = {
	{ "create", "no", shadow_types + 1 },
	{ "unset", "o", shadow_types + 3 },
	{ "destroy", "2", shadow_types + 0 },
};

WL_PRIVATE const struct wl_interface org_kde_kwin_shadow_manager_interface = {
	"org_kde_kwin_shadow_manager", 2,
	3, org_kde_kwin_shadow_manager_requests,
	0, NULL,
};

static const struct wl_message org_kde_kwin_shadow_requests[] = {
	{ "commit", "", shadow_types + 0 },
	{ "attach_left", "o", shadow_types + 4 },
	{ "attach_top_left", "o", shadow_types + 5 },
	{ "attach_top", "o", shadow_types + 6 },
	{ "attach_top_right", "o", shadow_types + 7 },
	{ "attach_right", "o", shadow_types + 8 },
	{ "attach_bottom_right", "o", shadow_types + 9 },
	{ "attach_bottom", "o", shadow_types + 10 },
	{ "attach_bottom_left", "o", shadow_types + 11 },
	{ "set_left_offset", "f", shadow_types + 0 },
	{ "set_top_offset", "f", shadow_types + 0 },
	{ "set_right_offset", "f", shadow_types + 0 },
	{ "set_bottom_offset", "f", shadow_types + 0 },
	{ "destroy", "2", shadow_types + 0 },
};

WL_PRIVATE const struct wl_interface org_kde_kwin_shadow_interface = {
	"org_kde_kwin_shadow", 2,
	14, org_kde_kwin_shadow_requests,
	0, NULL,
};

