// This file was generated by kcmutils_generate_module_data(): DO NOT EDIT!

#include "kwindecorationdata.h"

#include "kwindecorationsettings.h"




KWinDecorationData::KWinDecorationData(QObject *parent)
: KCModuleData(parent)
, m_settings(new KWinDecorationSettings(this))

{
autoRegisterSkeletons();
}

KWinDecorationSettings *KWinDecorationData::settings() const
{
    return m_settings;
}





#include "moc_kwindecorationdata.cpp"
